@@ -40,29 +40,36 @@ class Manager(object):
         self.undoaction = None
         self.redoaction = None
 
+    def clear(self):
+        self.states = []
+        self.label = None
+        self.current = 0
+
     def commit(self, label):
         """
         Must be called *BEFORE* each undoable actions
         :param label: label of the action
         """
         self.states = self.states[:self.current]
-        self.states.append(([list(row) for row in self.model], self.label,))
+        self.states.append(([row[0].duplicate(False) for row in self.model], self.label,))
         self.current += 1
         self.label = label
         self.__refresh()
 
     def undo(self, _action, _param, _unused):
         if self.current == len(self.states):
-            self.states.append(([list(row) for row in self.model], self.label,))
+            self.states.append(([row[0].duplicate(False) for row in self.model], self.label,))
         state, self.label = self.states[self.current - 1]
         self.__set_state(state)
         self.current -= 1
+        self.app.set_unsaved(True)
         self.__refresh()
 
     def redo(self, _action, _param, _unused):
         state, self.label = self.states[self.current + 1]
         self.__set_state(state)
         self.current += 1
+        self.app.set_unsaved(True)
         self.__refresh()
 
     def set_actions(self, undo, redo):
@@ -71,11 +78,20 @@ class Manager(object):
         self.__refresh()
 
     def __set_state(self, state):
-        self.model.clear()
-        for row in state:
-            # Do not reset the zoom level
-            row[4] = self.app.zoom_scale
-            self.model.append(row)
+        self.app.quit_rendering()
+        self.app.iconview.unselect_all()
+        with self.app.render_lock():
+            self.model.clear()
+            for page in state:
+                # Do not reset the zoom level
+                page.zoom = self.app.zoom_scale
+                page.resample = -1
+                self.model.append([page, page.description])
+        self.app.update_iconview_geometry()
+        self.app.update_max_zoom_level()
+        self.app.retitle()
+        self.app.update_statusbar()
+        self.app.silent_render()
 
     def __refresh(self):
         if self.undoaction:
